import React, { useEffect, useRef, useState } from 'react';
import { connect } from 'umi';
import { PageContainer } from '@ant-design/pro-layout';
import ProTable from '@ant-design/pro-table';
import { PlusOutlined } from '@ant-design/icons';
import { Button, Popconfirm, notification } from 'antd';
import {
  ModalForm,
  ProFormText,
} from '@ant-design/pro-form';
import request, { queryTable } from '@/utils/request';

const PostTypeList = (props) => {
  const tableRef = useRef()
  const form = useRef();
  const [current, setCurrent] = useState({})
  const [visible, setVisible] = useState(false)

  const handleEdit = (postType) => {
    const data = postType || {}
    setCurrent(data)
    setVisible(true)
    if (form.current) {
      form.current.setFieldsValue(data)
    }
  }

  const emitPublishOrNot = (postType) => {
    // 1 变成 0, 0 变成 1
    const status = Math.abs(postType.status - 1)
    request(`/post-type/${postType.typeId}`, { method: 'put', data: { ...postType, status } }).then(() => {
      notification.success({ message: '操作成功' })
      tableRef.current.reload()
    }).catch((e) => {
      notification.warn({ message: e.message })
    })
  }

  const handleSubmit = (postType) => {
    const method = current.typeId ? 'put' : 'post'
    const api = current.typeId ? `/post-type/${current.typeId}` : '/post-type'
    const data = { ...current, ...postType }
    return request(api, { method, data }).then(() => {
      notification.success({ message: '操作成功' })
      tableRef.current.reload()
      setVisible(false)
    }).catch((e) => {
      console.error(e)
      notification.warn({ message: e.message })
    })
  }

  const handleDelete = (row) => {
    return request(`/post-type/${row.typeId}`, { method: 'delete' }).then(() => {
      notification.success({ message: '操作成功' })
      tableRef.current.reload()
      setVisible(false)
    }).catch((e) => {
      console.error(e)
      notification.warn({ message: e.message })
    })
  }

  const actions = [
    <Button key="button" icon={<PlusOutlined />} type="primary" onClick={() => handleEdit()}>
      新建
    </Button>,
  ];

  const columns = [
    {
      title: '标题',
      key: 'name',
      dataIndex: 'name',
    },
    {
      title: '状态',
      key: 'status',
      dataIndex: 'status',
      width: 240,
      valueType: 'select',
      valueEnum: {
        0: {
          text: '下架',
          status: 'Processing',
        },
        1: {
          text: '上架',
          status: 'Success',
        },
      },
    },
    {
      title: '创建时间',
      key: 'createDate',
      dataIndex: 'createDate',
      valueType: 'date',
      width: 240,
      hideInSearch: true,
    },
    {
      title: '操作',
      key: 'option',
      valueType: 'option',
      width: 160,
      render: (_, item) => [
        <a key="edit" onClick={() => handleEdit(item)}>
          编辑
        </a>,
        <a key="up" onClick={() => emitPublishOrNot(item)}>
          {item.status === 1 ? '下架' : '上架'}
        </a>,
        <Popconfirm
          key="popconfirm"
          title={`确认删除吗?`}
          onConfirm={() => handleDelete(item)}
          okText="是"
          cancelText="否"
        >
          <a>删除</a>
        </Popconfirm>
      ],
    },
  ];

  useEffect(() => {
    if (!props.typeList || !props.typeList.length) {
      props.dispatch({
        type: 'post/getTypeList',
        payload: { pageSize: 999 },
      });
    }
  }, [props]);

  return (
    <PageContainer>
      <ProTable
        actionRef={tableRef}
        columns={columns}
        request={queryTable('/post-type')}
        rowKey="typeId"
        headerTitle="科普列表"
        search={{
          labelWidth: '4em',
        }}
        toolBarRender={() => actions}
      />
      <ModalForm formRef={form} width={600} visible={visible} onVisibleChange={setVisible} onFinish={handleSubmit}>
        <ProFormText
          label="名称"
          name="name"
          rules={[{ required: true, message: '请填写名称' }]}
        />
      </ModalForm>
    </PageContainer>
  );
};

export default connect((s) => ({
  typeList: s.post.typeList,
}))(PostTypeList);
